# Copyright 2021 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from glanceclient.common import utils
from glanceclient import exc

TARGET_VALUES = ('both', 'cache', 'queue')


class Controller(object):
    def __init__(self, http_client):
        self.http_client = http_client

    def is_supported(self, version):
        if utils.has_version(self.http_client, version):
            return True
        else:
            raise exc.HTTPNotImplemented(
                'Glance does not support image caching API (v2.14)')

    @utils.add_req_id_to_object()
    def list(self):
        if self.is_supported('v2.14'):
            url = '/v2/cache'
            resp, body = self.http_client.get(url)
            return body, resp

    @utils.add_req_id_to_object()
    def delete(self, image_id):
        if self.is_supported('v2.14'):
            resp, body = self.http_client.delete('/v2/cache/%s' %
                                                 image_id)
            return body, resp

    @utils.add_req_id_to_object()
    def clear(self, target):
        if self.is_supported('v2.14'):
            url = '/v2/cache'
            headers = {}
            if target != "both":
                headers = {'x-image-cache-clear-target': target}
            resp, body = self.http_client.delete(url, headers=headers)
            return body, resp

    @utils.add_req_id_to_object()
    def queue(self, image_id):
        if self.is_supported('v2.14'):
            url = '/v2/cache/%s' % image_id
            resp, body = self.http_client.put(url)
            return body, resp



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from glanceclient.common import http
from glanceclient.common import utils
from glanceclient.v2 import cache
from glanceclient.v2 import image_members
from glanceclient.v2 import image_tags
from glanceclient.v2 import images
from glanceclient.v2 import info
from glanceclient.v2 import metadefs
from glanceclient.v2 import schemas
from glanceclient.v2 import tasks
from glanceclient.v2 import versions


class Client(object):
    """Client for the OpenStack Images v2 API.

    :param string endpoint: A user-supplied endpoint URL for the glance
                            service.
    :param string token: Token for authentication.
    :param integer timeout: Allows customization of the timeout for client
                            http requests. (optional)
    :param string language_header: Set Accept-Language header to be sent in
                                   requests to glance.
    """

    def __init__(self, endpoint=None, **kwargs):
        self.endpoint_provided = endpoint is not None
        endpoint, self.version = utils.endpoint_version_from_url(endpoint, 2.0)
        self.http_client = http.get_http_client(endpoint=endpoint, **kwargs)
        self.schemas = schemas.Controller(self.http_client)

        self.images = images.Controller(self.http_client, self.schemas)
        self.image_tags = image_tags.Controller(self.http_client,
                                                self.schemas)
        self.image_members = image_members.Controller(self.http_client,
                                                      self.schemas)

        self.info = info.Controller(self.http_client, self.schemas)

        self.tasks = tasks.Controller(self.http_client, self.schemas)

        self.metadefs_resource_type = (
            metadefs.ResourceTypeController(self.http_client, self.schemas))

        self.metadefs_property = (
            metadefs.PropertyController(self.http_client, self.schemas))

        self.metadefs_object = (
            metadefs.ObjectController(self.http_client, self.schemas))

        self.metadefs_tag = (
            metadefs.TagController(self.http_client, self.schemas))

        self.metadefs_namespace = (
            metadefs.NamespaceController(self.http_client, self.schemas))

        self.versions = versions.VersionController(self.http_client)

        self.cache = cache.Controller(self.http_client)



# Copyright 2021 Red Hat Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools
from unittest import mock

from glanceclient.common import utils as common_utils
from glanceclient import exc
from glanceclient.tests import utils
from glanceclient.v2 import cache


data_fixtures = {
    '/v2/cache': {
        'GET': (
            {},
            {
                'cached_images': [
                    {
                        'id': 'b0aa672a-bc26-4fcb-8be1-f53ca361943d',
                        'Last Accessed (UTC)': '2021-08-09T07:08:20.214543',
                        'Last Modified (UTC)': '2021-08-09T07:08:20.214543',
                        'Size': 13267968,
                        'Hits': 0
                    },
                    {
                        'id': 'df601a47-7251-4d20-84ae-07de335af424',
                        'Last Accessed (UTC)': '2021-08-09T07:08:20.214543',
                        'Last Modified (UTC)': '2021-08-09T07:08:20.214543',
                        'Size': 13267968,
                        'Hits': 0
                    },
                ],
                'queued_images': [
                    '3a4560a1-e585-443e-9b39-553b46ec92d1',
                    '6f99bf80-2ee6-47cf-acfe-1f1fabb7e810'
                ],
            },
        ),
        'DELETE': (
            {},
            '',
        ),
    },
    '/v2/cache/3a4560a1-e585-443e-9b39-553b46ec92d1': {
        'PUT': (
            {},
            '',
        ),
        'DELETE': (
            {},
            '',
        ),
    },
}


class TestCacheController(testtools.TestCase):
    def setUp(self):
        super(TestCacheController, self).setUp()
        self.api = utils.FakeAPI(data_fixtures)
        self.controller = cache.Controller(self.api)

    @mock.patch.object(common_utils, 'has_version')
    def test_list_cached(self, mock_has_version):
        mock_has_version.return_value = True
        images = self.controller.list()
        # Verify that we have 2 cached and 2 queued images
        self.assertEqual(2, len(images['cached_images']))
        self.assertEqual(2, len(images['queued_images']))

    @mock.patch.object(common_utils, 'has_version')
    def test_list_cached_empty_response(self, mock_has_version):
        dummy_fixtures = {
            '/v2/cache': {
                'GET': (
                    {},
                    {
                        'cached_images': [],
                        'queued_images': [],
                    },
                ),
            }
        }
        dummy_api = utils.FakeAPI(dummy_fixtures)
        dummy_controller = cache.Controller(dummy_api)
        mock_has_version.return_value = True
        images = dummy_controller.list()
        # Verify that we have 0 cached and 0 queued images
        self.assertEqual(0, len(images['cached_images']))
        self.assertEqual(0, len(images['queued_images']))

    @mock.patch.object(common_utils, 'has_version')
    def test_queue_image(self, mock_has_version):
        mock_has_version.return_value = True
        image_id = '3a4560a1-e585-443e-9b39-553b46ec92d1'
        self.controller.queue(image_id)
        expect = [('PUT', '/v2/cache/%s' % image_id,
                   {}, None)]
        self.assertEqual(expect, self.api.calls)

    @mock.patch.object(common_utils, 'has_version')
    def test_cache_clear_with_header(self, mock_has_version):
        mock_has_version.return_value = True
        self.controller.clear("cache")
        expect = [('DELETE', '/v2/cache',
                   {'x-image-cache-clear-target': 'cache'}, None)]
        self.assertEqual(expect, self.api.calls)

    @mock.patch.object(common_utils, 'has_version')
    def test_cache_delete(self, mock_has_version):
        mock_has_version.return_value = True
        image_id = '3a4560a1-e585-443e-9b39-553b46ec92d1'
        self.controller.delete(image_id)
        expect = [('DELETE', '/v2/cache/%s' % image_id,
                   {}, None)]
        self.assertEqual(expect, self.api.calls)

    @mock.patch.object(common_utils, 'has_version')
    def test_cache_not_supported(self, mock_has_version):
        mock_has_version.return_value = False
        self.assertRaises(exc.HTTPNotImplemented,
                          self.controller.list)



# Copyright 2015 OpenStack Foundation
# Copyright 2015 Huawei Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


class VersionController(object):
    def __init__(self, http_client):
        self.http_client = http_client

    def list(self):
        """List all versions."""
        url = '/versions'
        resp, body = self.http_client.get(url)
        return body.get('versions', None)



# Copyright 2022 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools
from unittest import mock

from glanceclient.v2 import info


class TestController(testtools.TestCase):
    def setUp(self):
        super(TestController, self).setUp()
        self.fake_client = mock.MagicMock()
        self.info_controller = info.Controller(self.fake_client, None)

    def test_get_usage(self):
        fake_usage = {
            'usage': {
                'quota1': {'limit': 10, 'usage': 0},
                'quota2': {'limit': 20, 'usage': 5},
            }
        }
        self.fake_client.get.return_value = (mock.MagicMock(), fake_usage)
        usage = self.info_controller.get_usage()
        self.assertEqual(fake_usage['usage'], usage)
        self.fake_client.get.assert_called_once_with('/v2/info/usage')
